<?php
// +---------------------------------------------------------------------------+
// |                         PHP::XTPL  Template Engine                        |
// +---------------------------------------------------------------------------+
// | Ultimate XTHML tag class.                                                 |
// +---------------------------------------------------------------------------+
// |  Written by Vladimir Sibirov a.k.a. Trustmaster.                          |
// |  e-mail: trustware@bk.ru                                                  |
// +---------------------------------------------------------------------------+

/**
* Ultimate PHP implementation of XTHML tag. This can be used as wrapper in your PHP code
* to connect the page code with the template. Or extend this class by any of yours.
*
* @access public
* @version XTPL, v.1.2
* @package XTPL
* @author Vladimir Sibirov <trustware@bk.ru>
*/
class HtmlTag
{
        var $Attr = Array();
        var $tag = '';
        var $content = null;

        /**
        * Constructor: creates new HtmlTag object. If $content argument is not passed, the tag won't have
        * closing one (<tag />). Otherwise $content may contain any string, int, etc. It is also allowed
        * to insert HtmlTag objects into it. It is the way of tag nesting.
        * All the tags have attributes. You can access them using accociative array HtmlTag::Attr.
        *
        * Example:
        *
        * $br = new HtmlTag('br');
        * $h1 = new HtmlTag('h1', 'Some text here');
        * $link = new HtmlTag('a', 'LocalHost');
        * $link->Attr['href'] = 'http://localhost';
        * $link->Attr['target'] = '_blank';
        * $h1_link = new HtmlTag('h1', $link);
        *
        * @access public
        * @return HtmlTag
        * @param string $tag Tag name. e.g. 'img'
        * @param mixed $content Tag content.
        */
        function HtmlTag($tag, $content = null)
        {
                $this->content = $content;
                $this->tag = $tag;
        }
        
        /**
        * Inserts another HtmlTag into this one. $internal_tag may also contain some value.
        * This is used for implementation of tag nesting.
        *
        * Example:
        *
        * // Creating select tag using Insert method
        * $select = new HtmlTag('select');
        * $select->Attr['name'] = 'user';
        * for($i = 0; $i < count($users); $i++)
        * {
        * 	$option = new HtmlTag('option', $users[$i]);
        * 	$option->Attr['value'] = $i;
        * 	$select->Insert($option);
        * }
        * // Sending the tag to output
        * $select->Parse();
        *
        *
        * @access public
        * @return void
        * @param mixed $internal_tag Object for insertion.
        */
        function Insert($internal_tag)
        {
                if(is_null($this->content))
                {
                	$this->content = $internal_tag;
                }
        	elseif(!is_array($this->content))
                {
                        if(!is_null($this->content)) $tmp = $this->content;
                        $this->content = Array();
                        if(!is_null($tmp)) $this->content[0] = $tmp;
                        array_push($this->content, $internal_tag);
                }
                else array_push($this->content, $internal_tag);
        }
        
        /**
        * Removes tag's content element.
        *
        * @access public
        * @return void
        * @param int $index Position inside the tag.
        */
        function Remove($index)
        {
                unset($this->content[$index]);
        }
        
        /**
        * Renders the object into XHTML code string.
        *
        * Example:
        *
        * $h1 = new HtmlTag('h1', 'some text');
        * $code = $h1->GetCode();
        * // The result is: <h1>some text</h1>
        *
        * @access public
        * @return string
        */
        function GetCode()
        {
                $code = '<'.$this->tag;
                foreach($this->Attr as $key=>$val)
                {
                        if($val != '') $code .= ' '.$key.'="'.$val.'"';
                }
                if(is_null($this->content)) $code .= ' />';
                elseif(!is_object($this->content) && !is_array($this->content)) $code .= '>'.$this->content.'</'.$this->tag.'>';
                elseif(is_object($this->content)) $code .= '>'.$this->content->GetCode().'</'.$this->tag.'>';
                elseif(is_array($this->content))
                {
                        $code .= '>';
                        for($i = 0; $i<count($this->content); $i++)
                        {
                                if(is_object($this->content[$i])) $code .= $this->content[$i]->GetCode();
                                elseif(!is_null($this->content[$i])) $code .= $this->content[$i];
                        }
                        $code .= '</'.$this->tag.'>';
                }
                return $code;
        }
        
        /**
        * Renders the object and sends the result XHTML code to output.
        *
        * Example:
        *
        * // Creating select tag using Insert method
        * $select = new HtmlTag('select');
        * $select->Attr['name'] = 'user';
        * for($i = 0; $i < count($users); $i++)
        * {
        * 	$option = new HtmlTag('option', $users[$i]);
        * 	$option->Attr['value'] = $i;
        * 	$select->Insert($option);
        * }
        * // Sending the tag to output
        * $select->Parse();
        *
        * @access public
        * @return void
        */
        function Parse()
        {
                echo $this->GetCode();
        }
}
?>