<?php
// +---------------------------------------------------------------------------+
// |                         PHP::XTPL  Template Engine                        |
// +---------------------------------------------------------------------------+
// | XTPL Template Core class.                                                 |
// +---------------------------------------------------------------------------+
// |  Written by Vladimir Sibirov a.k.a. Trustmaster.                          |
// |  e-mail: trustware@bk.ru                                                  |
// +---------------------------------------------------------------------------+

define('ID_CHARS', '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_');
define('VAR_CHARS', '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_');

/**
* XTPL template class. This is the main XTPL class and it provides you
* easy-to-use template engine.
*
* @version XTPL, v.1.2
* @access public
* @package XTPL
* @author Vladimir Sibirov <trustware@bk.ru>
*/
class XTPL
{
	var $vars = Array();
	var $gzip = false;

	/**
	* Constructor: creates new XTPL object.
	* It turns GZIP output compression on if XTPL_GZIP constant is true
	*
	* Example:
	*
	* define('XTPL_GZIP', true);
	* $page = new XTPL();
	*
	* @access public
	* @return XTPL
	*/
	function XTPL()
	{
		if(defined('XTPL_GZIP') && XTPL_GZIP)
		{
			$this->gzip = true;
		}
	}

	/**
	* Starts output buffering. You may control GZIP comperession of the ouput by the argument.
	* Output buffering allows you to send any headers, cookies and use session_ functions until
	* the buffer is flushed. It also allows you to use GZIP output compression which helps to
	* compress the page size if the browser supports the compression (or the content is sent
	* in standard way if GZIP compression is not supported).
	*
	* Example:
	*
	* $page = new XTPL();
	* $page->Start(true);
	* $page->Parse('tpl/template.tpl');
	* $page->End();
	*
	* @access public
	* @return void
	* @param bool $gzip Output ZLib compression
	*/
	function Start($gzip = false)
	{
		if($gzip) $this->gzip = true;
		if($this->gzip) ob_start('ob_gzhandler');
		else ob_start();
	}

	/**
	* Immediately flushes the output buffer.
	*
	* Example:
	*
	* $page = new XTPL();
	* $page->Start();
	* $page->Parse('tpl/template.tpl');
	* // flushing the buffer before another template is parsed
	* $page->Flush();
	* $page->Parse('tpl/another.tpl');
	* // End output buffering
	* $page->End();
	*
	* @access public
	* @return void
	*/
	function Flush()
	{
		ob_flush();
	}

	/**
	* Flushes the output buffer and switches output buffering off.
	*
	* Example:
	*
	* $page->End();
	* // Now printing immediately
	* echo 'Hi, can you see this?';
	*
	* @access public
	* @return void
	*/
	function End()
	{
		ob_end_flush();
	}

	/**
	* Sets the value of output buffer GZIP compression to $value if it is defined.
	* Otherwise returns current value of output buffer ZLib compression.
	*
	* Example:
	*
	* $current_value = $page->GZIP();
	* if(!$current_value) $page->GZIP(true);
	*
	* @access public
	* @return bool
	* @param bool $value Whether to switch GZIP compression on/off.
	*/
	function GZIP($value = null)
	{
		if(!is_null($value)) $this->gzip = $value;
		return $this->gzip;
	}

	/**
	* Assigns a new private XTPL variable.
	* When XTPL meet {$var} sequence it first looks for the replacement in
	* the private section. If the variable doesn't exist in the private section
	* then XTPL searches for it in the global scope.
	*
	* Example:
	*
	* $id = 25;
	* $page->ParseStr('id is {$id}');
	* // 25 is printed
	* $page->Assign('id', 10);
	* $page->ParseStr('id is {$id}');
	* // 10 is printed
	*
	* @access public
	* @return void
	* @param string $name Name of new variable.
	* @param mixed $value Value of the variable.
	*/
	function Assign($name, $value)
	{
		$this->vars[$name] = $value;
	}
	
	/**
	* Removes the variable from the private assigned list. Unassignes the variable which has been
	* assigned by XTPL::Assign() method.
	*
	* @return void
	* @param string $name Name of variable to remove.
	*/
	function Unassign($name)
	{
		unset($this->vars[$name]);
	}

	/**
	* Parses string and sends result to output. You may use this function to parse the strings
	* with XTPL syntax directly but it is not recommended.
	*
	* @access public
	* @return void
	* @param string $str_var String to parse.
	*/
	function ParseStr($content)
	{
		if(!strstr($content, '{') || !strstr($content, '}')) echo $content;
		else
		{
			$postfix = substr($content, strrpos($content, '}') + 1, strlen($content) - strrpos($content, '}') - 1);
			$content = substr($content, 0, strrpos($content, '}') + 1);
			do
			{
				$regs = Array();
				$done = Array();
				$start = strpos($content, '{');
				echo substr($content, 0, $start);
				$content = substr($content, $start, strlen($content) - $start);
				$seq = substr($content, 0, strpos($content, '}') + 1);
				$content = substr($content, strlen($seq), strlen($content) - strlen($seq));
				if($seq[1] == '/')
				{
					$seq_closing = true;
					$seq_type = substr($seq, 2, strlen($seq) - 3);
				}
				elseif($seq[1] == '$')
				{
					$seq_type = '$';
				}
				else
				{
					if(strstr($seq, ' ')) $seq_type = substr($seq, 1, strpos($seq, ' ') - 1);
					else $seq_type = substr($seq, 1, strlen($seq) - 2);
				}
				$seq_closed = ($seq[strlen($seq) - 2] == '/') ? true : false;
				$seq_len = strlen($seq);
				if($seq_type == '$')
				{
					// Variable sequence
					$var = substr($seq, 1, strlen($seq) - 2);
					preg_match_all('/(\$[a-z0-9_]+)/i', $var, $regs);
					for($j = 0; $j < count($regs[0]); $j++)
					{
						$var_name = str_replace('$', '', $regs[0][$j]);
						if(!in_array($var_name, $done))
						{
							if(isset($this->vars[$var_name])) $var = str_replace('$'.$var_name, "\$this->vars['$var_name']", $var);
							else global ${$var_name};
							$done[] = $var_name;
						}
					}
					if(strstr($var, '[$'))
					{
						$var_name = substr($var, strpos($var, '[$') + 2, strpos($var, ']') - strpos($var, '[$') - 1);
						if(isset($this->vars[$var_name])) $var = str_replace($var_name, "this->vars[$var_name]", $var);
						else global ${$var_name};
					}
					if(strstr($var, ':') && (!strstr($var, '|')))
					{
						$var = substr($var, 0, strpos($var, ':'));
						eval("$var".'->Parse();');
					}
					elseif(strstr($var, '|'))
					{
						$var_arr = explode('|', $var);
						eval("\$temp = $var_arr[0];");
						for($j = 1; $j < count($var_arr); $j++)
						{
							$func_name = trim($var_arr[$j]);
							if(strstr($func_name, '('))
							{
								if(!strstr($var_arr[$j], $var_arr[0]))
								{
									$func_parts = explode('(', $func_name);
									$func_name = $func_parts[0].'($temp, '.$func_parts[1];
								}
								else $func_name = str_replace($var_arr[0], '$temp', $func_name);
								eval("\$temp = $func_name;");
							}
							else
							{
								eval("\$temp = $func_name(\$temp);");
							}
						}
						echo $temp;
					}
					else eval("echo $var;");
				}
				elseif($seq_type == 'include')
				{
					// Include sequence
					if(strstr($seq, '"')) $fname = substr($seq, strpos($seq, '"') + 1, strrpos($seq, '"') - strpos($seq, '"') - 1);
					else $fname = substr($seq, strpos($seq, ' ') + 1, strlen($seq) - strpos($seq, ' ') - 2);
					$this->Parse($fname);
				}
				elseif($seq_type == 'if')
				{
					// Logical expression sequence
					$var = substr($seq, strpos($seq, ' ') + 1, strpos($seq, '}') - strpos($seq, ' ') - 1);
					$code = '';
					$seqcount = 1;
					do
					{
						$cl_pos = strpos($content, '{/'.$seq_type.'}');
						$op_pos = strpos($content, '{'.$seq_type);
						if(($op_pos >= 0) && ($op_pos < $cl_pos))
						{
							$code .= substr($content, 0, $op_pos);
							$content = substr($content, $op_pos, strlen($content) - $op_pos);
							$code .= substr($content, 0, strpos($content, '}') + 1);
							$content = substr($content, strpos($content, '}') + 1, strlen($content) - strpos($content, '}') - 1);
							$seqcount++;
						}
						else
						{
							$code .= substr($content, 0, $cl_pos + strlen('{/'.$seq_type.'}'));
							$content = substr($content, $cl_pos + strlen('{/'.$seq_type.'}'), strlen($content) - $cl_pos - strlen('{/'.$seq_type.'}'));
							$seqcount--;
						}
					}
					while($seqcount > 0);
					// Trimming the last occurance of closing tag
					$code = substr($code, 0, strrpos($code, '{/'.$seq_type.'}'));
					preg_match_all('/(\$[a-z0-9_]+)/i', $var, $regs);
					for($j = 0; $j < count($regs[0]); $j++)
					{
						$var_name = str_replace('$', '', $regs[0][$j]);
						if(!in_array($var_name, $done))
						{
							if(isset($this->vars[$var_name])) $var = str_replace('$'.$var_name, "\$this->vars['$var_name']", $var);
							else global ${$var_name};
							$done[] = $var_name;
						}
					}
					eval("\$check_var = ($var) ? true : false;");
					if(strstr($code, '{else}'))
					{
						$code_array[0] = substr($code, 0, strrpos($code, '{else}'));
						$code_array[1] = substr($code, strrpos($code, '{else}') + 6, strlen($code) - strrpos($code, '{else}') - 6);
						$check_var ? $this->ParseStr($code_array[0]) : $this->ParseStr($code_array[1]);
					}
					else
					{
						if($check_var) $this->ParseStr($code);
					}
				}
				elseif($seq_type == 'modify')
				{
					// Modifier sequence
					$code = substr($content, 0, strpos($content, '{/modify}') - $i);
					$content = substr($content, strlen($code) + 9, strlen($content) - strlen($code) - 9);
					$seq_args = explode('|', substr($seq, strpos($seq, ' '), strlen($seq) - strpos($seq, ' ') - 1));
					for($j = 0; $j < count($seq_args); $j++)
					{
						$func_name = trim($seq_args[$j]);
						if(strstr($func_name, '('))
						{
							if(!strstr($func_name, '$content'))
							{
								$func_parts = explode('(', $func_name);
								$func_name = $func_parts[0].'($code, '.$func_parts[1];
							}
							else $func_name = str_replace('$content', '$code', $func_name);
							eval("\$code = $func_name;");
						}
						else
						{
							eval("\$code = $func_name(\$code);");
						}
					}
					echo $code;
				}
				elseif($seq_type == 'for')
				{
					// Cycle for sequence
					$code = '';
					$seqcount = 1;
					do
					{
						$cl_pos = strpos($content, '{/'.$seq_type.'}');
						$op_pos = strpos($content, '{'.$seq_type);
						if(($op_pos >= 0) && ($op_pos < $cl_pos))
						{
							$code .= substr($content, 0, $op_pos);
							$content = substr($content, $op_pos, strlen($content) - $op_pos);
							$code .= substr($content, 0, strpos($content, '}') + 1);
							$content = substr($content, strpos($content, '}') + 1, strlen($content) - strpos($content, '}') - 1);
							$seqcount++;
						}
						else
						{
							$code .= substr($content, 0, $cl_pos + strlen('{/'.$seq_type.'}'));
							$content = substr($content, $cl_pos + strlen('{/'.$seq_type.'}'), strlen($content) - $cl_pos - strlen('{/'.$seq_type.'}'));
							$seqcount--;
						}
					}
					while($seqcount > 0);
					// Trimming the last occurance of closing tag
					$code = substr($code, 0, strrpos($code, '{/'.$seq_type.'}'));
					$var = substr($seq, 5, strlen($seq) - 6);
					$key_id = substr($var, strpos($var, '$') + 1, strpos($var, ';') - strpos($var, '$'));
					if(stristr($key_id, ' ')) $key_id = substr($key_id, 0, strpos($key_id, ' '));
					else $key_id = substr($key_id, 0, strpos($key_id, '='));
					$this->Assign($key_id, 0);
					preg_match_all('/(\$[a-z0-9_]+)/i', $var, $regs);
					for($j = 0; $j < count($regs[0]); $j++)
					{
						$var_name = str_replace('$', '', $regs[0][$j]);
						if(!in_array($var_name, $done))
						{
							if(isset($this->vars[$var_name])) $var = str_replace('$'.$var_name, "\$this->vars['$var_name']", $var);
							else global ${$var_name};
							$done[] = $var_name;
						}
					}
					eval("for($var) \$this->ParseStr(\$code);");
				}
				elseif($seq_type == 'while')
				{
					// Cycle while sequence
					$code = '';
					$seqcount = 1;
					do
					{
						$cl_pos = strpos($content, '{/'.$seq_type.'}');
						$op_pos = strpos($content, '{'.$seq_type);
						if(($op_pos >= 0) && ($op_pos < $cl_pos))
						{
							$code .= substr($content, 0, $op_pos);
							$content = substr($content, $op_pos, strlen($content) - $op_pos);
							$code .= substr($content, 0, strpos($content, '}') + 1);
							$content = substr($content, strpos($content, '}') + 1, strlen($content) - strpos($content, '}') - 1);
							$seqcount++;
						}
						else
						{
							$code .= substr($content, 0, $cl_pos + strlen('{/'.$seq_type.'}'));
							$content = substr($content, $cl_pos + strlen('{/'.$seq_type.'}'), strlen($content) - $cl_pos - strlen('{/'.$seq_type.'}'));
							$seqcount--;
						}
					}
					while($seqcount > 0);
					// Trimming the last occurance of closing tag
					$code = substr($code, 0, strrpos($code, '{/'.$seq_type.'}'));
					$var = substr($seq, 7, strlen($seq) - 8);
					preg_match_all('/(\$[a-z0-9_]+)/i', $var, $regs);
					for($j = 0; $j < count($regs[0]); $j++)
					{
						$var_name = str_replace('$', '', $regs[0][$j]);
						if(!in_array($var_name, $done))
						{
							if(isset($this->vars[$var_name])) $var = str_replace('$'.$var_name, "\$this->vars['$var_name']", $var);
							else global ${$var_name};
							$done[] = $var_name;
						}
					}
					eval("while($var) \$this->ParseStr(\$code);");
				}
				elseif($seq_type == 'php')
				{
					// Code evalation
					$code = '';
					$seqcount = 1;
					do
					{
						$cl_pos = strpos($content, '{/'.$seq_type.'}');
						$op_pos = strpos($content, '{'.$seq_type);
						if(($op_pos >= 0) && ($op_pos < $cl_pos))
						{
							$code .= substr($content, 0, $op_pos);
							$content = substr($content, $op_pos, strlen($content) - $op_pos);
							$code .= substr($content, 0, strpos($content, '}') + 1);
							$content = substr($content, strpos($content, '}') + 1, strlen($content) - strpos($content, '}') - 1);
							$seqcount++;
						}
						else
						{
							$code .= substr($content, 0, $cl_pos + strlen('{/'.$seq_type.'}'));
							$content = substr($content, $cl_pos + strlen('{/'.$seq_type.'}'), strlen($content) - $cl_pos - strlen('{/'.$seq_type.'}'));
							$seqcount--;
						}
					}
					while($seqcount > 0);
					// Trimming the last occurance of closing tag
					$code = substr($code, 0, strrpos($code, '{/'.$seq_type.'}'));
					preg_match_all('/(\$[a-z0-9_]+)/i', $code, $regs);
					for($j = 0; $j < count($regs[0]); $j++)
					{
						$var_name = str_replace('$', '', $regs[0][$j]);
						if(!in_array($var_name, $done))
						{
							if(isset($this->vars[$var_name])) $code = str_replace('$'.$var_name, "\$this->vars['$var_name']", $code);
							else global ${$var_name};
							$done[] = $var_name;
						}
					}
					eval($code);
				}
				elseif($seq_type == '*')
				{
					// Comment sequence
				}
				else
				{
					// Otherwise the sequence is ignored
					echo $seq;
				}
			}
			while(strlen($content) > 0);
			echo $postfix;
		}
	}

	/**
	* Parses the template file.
	*
	* Example:
	*
	* $page->Parse('template.tpl');
	*
	* @access public
	* @return void
	* @param string $filename Template file name.
	*/
	function Parse($filename)
	{
		if(!file_exists($filename))
		{
			echo "XTPL Error: Template doesn\'t exist ($filename).";
			exit;
		}
		$content = file_get_contents($filename);
		$start = strpos($content, '{');
		$end = strrpos($content, '}');
		echo substr($content, 0, $start);
		$this->ParseStr(substr($content, $start, ($end - $start) + 1));
		echo substr($content, $end + 1, strlen($content) - $end);
	}
}
?>